"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@anycli/command");
const Octokit = require("@octokit/rest");
const cli_ux_1 = require("cli-ux");
const fs = require("fs-extra");
const qq = require("qqjs");
const project_1 = require("../project");
const debug = require('debug')('release');
class Release extends command_1.Command {
    async run() {
        if (!process.env.GH_TOKEN)
            throw new Error('GH_TOKEN must be set');
        const { flags } = this.parse(Release);
        this.info = project_1.info();
        const octokit = new Octokit();
        octokit.authenticate({
            type: 'token',
            token: process.env.GH_TOKEN,
        });
        const tag = `v${this.info.version}`;
        let release;
        try {
            const { data } = await octokit.repos.createRelease({
                owner: 'jdxcode',
                repo: 'bin-it',
                target_commitish: this.info.sha,
                tag_name: tag,
                prerelease: flags.prerelease,
                draft: flags.draft,
            });
            debug(data);
            release = data;
        }
        catch (err) {
            debug(err);
            // try getting a release if we can't create one
            const { data } = await octokit.repos.getReleaseByTag({
                owner: 'jdxcode',
                repo: 'bin-it',
                tag,
            });
            release = data;
        }
        const assets = qq.globby([`dist/${this.info.version}/${this.info.name}-v${this.info.version}*.tar.gz`]);
        debug(assets);
        for (let file of assets) {
            const result = await octokit.repos.uploadAsset({
                url: release.upload_url,
                file: fs.createReadStream(file),
                contentType: 'application/gzip',
                contentLength: fs.statSync(file).size,
                name: qq.path.basename(file),
                label: qq.path.basename(file),
            });
            debug(result);
        }
    }
    async catch(err) {
        if (err.response && err.response.data && err.response.data.errors) {
            for (let e of err.response.data.errors) {
                cli_ux_1.default.warn(e);
            }
        }
        super.catch(err);
    }
}
Release.flags = {
    // channel: flags.string({required: true}),
    prerelease: command_1.flags.boolean(),
    draft: command_1.flags.boolean(),
};
exports.default = Release;
